@@ -1,7 +1,7 @@
 /*
  * wee-completion.c - completion for WeeChat commands
  *
- * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2006 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -46,6 +46,8 @@
 #include "wee-string.h"
 #include "../gui/gui-completion.h"
 #include "../gui/gui-bar.h"
+#include "../gui/gui-bar-item.h"
+#include "../gui/gui-bar-item-custom.h"
 #include "../gui/gui-bar-window.h"
 #include "../gui/gui-buffer.h"
 #include "../gui/gui-color.h"
@@ -60,6 +62,28 @@
 extern char **environ;
 
 
+/*
+ * Adds a word with quotes around to completion list.
+ */
+
+void
+completion_list_add_quoted_word (struct t_gui_completion *completion,
+                                 const char *word)
+{
+    char *temp;
+    int length;
+
+    length = 1 + strlen (word) + 1 + 1;
+    temp = malloc (length);
+    if (!temp)
+        return;
+
+    snprintf (temp, length, "\"%s\"", word);
+    gui_completion_list_add (completion, temp, 0, WEECHAT_LIST_POS_END);
+
+    free (temp);
+}
+
 /*
  * Adds bar names to completion list.
  */
@@ -87,6 +111,152 @@ completion_list_add_bars_names_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Adds custom bar items names to completion list.
+ */
+
+int
+completion_list_add_custom_bar_items_names_cb (const void *pointer, void *data,
+                                               const char *completion_item,
+                                               struct t_gui_buffer *buffer,
+                                               struct t_gui_completion *completion)
+{
+    struct t_gui_bar_item_custom *ptr_item;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    for (ptr_item = gui_custom_bar_items; ptr_item;
+         ptr_item = ptr_item->next_item)
+    {
+        gui_completion_list_add (completion, ptr_item->bar_item->name,
+                                 0, WEECHAT_LIST_POS_SORT);
+    }
+
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Adds custom bar item conditions to completion list.
+ */
+
+int
+completion_list_add_custom_bar_item_conditions_cb (const void *pointer, void *data,
+                                                   const char *completion_item,
+                                                   struct t_gui_buffer *buffer,
+                                                   struct t_gui_completion *completion)
+{
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    gui_completion_list_add (completion,
+                             "\"" GUI_BAR_ITEM_CUSTOM_DEFAULT_CONDITIONS "\"",
+                             0,
+                             WEECHAT_LIST_POS_END);
+    gui_completion_list_add (completion, "\"\"", 0, WEECHAT_LIST_POS_END);
+
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Adds custom bar item contents to completion list.
+ */
+
+int
+completion_list_add_custom_bar_item_contents_cb (const void *pointer, void *data,
+                                                 const char *completion_item,
+                                                 struct t_gui_buffer *buffer,
+                                                 struct t_gui_completion *completion)
+{
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    gui_completion_list_add (completion,
+                             "\"" GUI_BAR_ITEM_CUSTOM_DEFAULT_CONTENTS "\"",
+                             0,
+                             WEECHAT_LIST_POS_END);
+    gui_completion_list_add (completion, "\"\"", 0, WEECHAT_LIST_POS_END);
+
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Adds arguments for commands that add a custom bar item.
+ */
+
+int
+completion_list_add_custom_bar_item_add_arguments_cb (const void *pointer, void *data,
+                                                      const char *completion_item,
+                                                      struct t_gui_buffer *buffer,
+                                                      struct t_gui_completion *completion)
+{
+    char **sargv;
+    int sargc, arg_complete;
+    struct t_gui_bar_item_custom *ptr_item;
+
+    if (!completion->args)
+        return WEECHAT_RC_OK;
+
+    sargv = string_split_shell (completion->args, &sargc);
+    if (!sargv)
+        return WEECHAT_RC_OK;
+
+    ptr_item = (sargc > 1) ? gui_bar_item_custom_search (sargv[1]) : NULL;
+
+    arg_complete = sargc;
+    if (completion->base_word && completion->base_word[0])
+        arg_complete--;
+
+    switch (arg_complete)
+    {
+        case 1:
+            completion_list_add_custom_bar_items_names_cb (pointer, data,
+                                                           completion_item,
+                                                           buffer, completion);
+
+            break;
+        case 2:
+            if (ptr_item)
+            {
+                completion_list_add_quoted_word (
+                    completion,
+                    CONFIG_STRING(ptr_item->options[GUI_BAR_ITEM_CUSTOM_OPTION_CONDITIONS]));
+            }
+            else
+            {
+                completion_list_add_custom_bar_item_conditions_cb (
+                    pointer, data, completion_item, buffer, completion);
+            }
+            break;
+        case 3:
+            if (ptr_item)
+            {
+                completion_list_add_quoted_word (
+                    completion,
+                    CONFIG_STRING(ptr_item->options[GUI_BAR_ITEM_CUSTOM_OPTION_CONTENT]));
+            }
+            else
+            {
+                completion_list_add_custom_bar_item_contents_cb (
+                    pointer, data, completion_item, buffer, completion);
+            }
+            break;
+    }
+
+    string_free_split (sargv);
+
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Adds bar options to completion list.
  */
@@ -319,6 +489,76 @@ completion_list_add_buffer_properties_set_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Adds a buffer local variable to completions list (for `/buffer setauto`).
+ */
+
+void
+completion_list_map_buffer_local_variable_setauto_cb (void *data,
+                                                      struct t_hashtable *hashtable,
+                                                      const void *key,
+                                                      const void *value)
+{
+    char str_localvar[4096];
+
+    /* make C compiler happy */
+    (void) hashtable;
+    (void) value;
+
+    snprintf (str_localvar, sizeof (str_localvar),
+              "localvar_set_%s", (const char *)key);
+    gui_completion_list_add ((struct t_gui_completion *)data,
+                             str_localvar,
+                             0, WEECHAT_LIST_POS_SORT);
+}
+
+/*
+ * Adds buffer properties (that can be set), local variables and key bindings
+ * to completion list.
+ */
+
+int
+completion_list_add_buffer_properties_setauto_cb (const void *pointer, void *data,
+                                                  const char *completion_item,
+                                                  struct t_gui_buffer *buffer,
+                                                  struct t_gui_completion *completion)
+{
+    struct t_gui_key *ptr_key;
+    char str_key[1024];
+    int i;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    /* add buffer properties */
+    for (i = 0; gui_buffer_properties_set[i]; i++)
+    {
+        gui_completion_list_add (completion,
+                                 gui_buffer_properties_set[i],
+                                 0, WEECHAT_LIST_POS_SORT);
+    }
+
+    /* add buffer local variables */
+    hashtable_map (completion->buffer->local_variables,
+                   &completion_list_map_buffer_local_variable_setauto_cb,
+                   completion);
+
+    /* add buffer keys */
+    for (ptr_key = completion->buffer->keys; ptr_key;
+         ptr_key = ptr_key->next_key)
+    {
+        snprintf (str_key, sizeof (str_key), "key_bind_%s", ptr_key->key);
+        gui_completion_list_add (completion, str_key, 0, WEECHAT_LIST_POS_SORT);
+        snprintf (str_key, sizeof (str_key), "key_unbind_%s", ptr_key->key);
+        gui_completion_list_add (completion, str_key, 0, WEECHAT_LIST_POS_SORT);
+    }
+
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Adds buffer properties (that can be read) to completion list.
  */
@@ -691,6 +931,68 @@ completion_list_add_filters_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Adds disabled filter names to completion list.
+ */
+
+int
+completion_list_add_filters_disabled_cb (const void *pointer, void *data,
+                                         const char *completion_item,
+                                         struct t_gui_buffer *buffer,
+                                         struct t_gui_completion *completion)
+{
+    struct t_gui_filter *ptr_filter;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    for (ptr_filter = gui_filters; ptr_filter;
+         ptr_filter = ptr_filter->next_filter)
+    {
+        if (!ptr_filter->enabled)
+        {
+            gui_completion_list_add (completion, ptr_filter->name,
+                                     0, WEECHAT_LIST_POS_SORT);
+        }
+    }
+
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Adds enabled filter names to completion list.
+ */
+
+int
+completion_list_add_filters_enabled_cb (const void *pointer, void *data,
+                                        const char *completion_item,
+                                        struct t_gui_buffer *buffer,
+                                        struct t_gui_completion *completion)
+{
+    struct t_gui_filter *ptr_filter;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    for (ptr_filter = gui_filters; ptr_filter;
+         ptr_filter = ptr_filter->next_filter)
+    {
+        if (ptr_filter->enabled)
+        {
+            gui_completion_list_add (completion, ptr_filter->name,
+                                     0, WEECHAT_LIST_POS_SORT);
+        }
+    }
+
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Adds command hooks to completion list.
  */
@@ -1092,7 +1394,7 @@ completion_list_add_plugins_commands_cb (const void *pointer, void *data,
             arg_index = argc - 1;
 
         ptr_plugin = NULL;
-        if (string_strcasecmp (argv[arg_index], PLUGIN_CORE) != 0)
+        if (strcmp (argv[arg_index], PLUGIN_CORE) != 0)
         {
             /*
              * plugin name is different from "core", then search it in
@@ -1229,59 +1531,31 @@ completion_list_add_config_option_values_cb (const void *pointer, void *data,
                             }
                             break;
                         case CONFIG_OPTION_TYPE_INTEGER:
-                            if (option_found->string_values)
-                            {
-                                for (i = 0; option_found->string_values[i]; i++)
-                                {
-                                    gui_completion_list_add (completion,
-                                                             option_found->string_values[i],
-                                                             0, WEECHAT_LIST_POS_SORT);
-                                }
-                                gui_completion_list_add (completion, "++1",
-                                                         0, WEECHAT_LIST_POS_END);
+                            if (option_found->value && CONFIG_INTEGER(option_found) > option_found->min)
                                 gui_completion_list_add (completion, "--1",
-                                                         0, WEECHAT_LIST_POS_END);
-                                if (option_found->value)
+                                                         0, WEECHAT_LIST_POS_BEGINNING);
+                            if (option_found->value && CONFIG_INTEGER(option_found) < option_found->max)
+                                gui_completion_list_add (completion, "++1",
+                                                         0, WEECHAT_LIST_POS_BEGINNING);
+                            if (option_found->value)
+                            {
+                                length = 64;
+                                value_string = malloc (length);
+                                if (value_string)
                                 {
+                                    snprintf (value_string, length,
+                                              "%d", CONFIG_INTEGER(option_found));
                                     gui_completion_list_add (completion,
-                                                             option_found->string_values[CONFIG_INTEGER(option_found)],
-                                                             0, WEECHAT_LIST_POS_BEGINNING);
-                                }
-                                else
-                                {
-                                    gui_completion_list_add (completion,
-                                                             WEECHAT_CONFIG_OPTION_NULL,
+                                                             value_string,
                                                              0, WEECHAT_LIST_POS_BEGINNING);
+                                    free (value_string);
                                 }
                             }
                             else
                             {
-                                if (option_found->value && CONFIG_INTEGER(option_found) > option_found->min)
-                                    gui_completion_list_add (completion, "--1",
-                                                             0, WEECHAT_LIST_POS_BEGINNING);
-                                if (option_found->value && CONFIG_INTEGER(option_found) < option_found->max)
-                                    gui_completion_list_add (completion, "++1",
-                                                             0, WEECHAT_LIST_POS_BEGINNING);
-                                if (option_found->value)
-                                {
-                                    length = 64;
-                                    value_string = malloc (length);
-                                    if (value_string)
-                                    {
-                                        snprintf (value_string, length,
-                                                  "%d", CONFIG_INTEGER(option_found));
-                                        gui_completion_list_add (completion,
-                                                                 value_string,
-                                                                 0, WEECHAT_LIST_POS_BEGINNING);
-                                        free (value_string);
-                                    }
-                                }
-                                else
-                                {
-                                    gui_completion_list_add (completion,
-                                                             WEECHAT_CONFIG_OPTION_NULL,
-                                                             0, WEECHAT_LIST_POS_BEGINNING);
-                                }
+                                gui_completion_list_add (completion,
+                                                         WEECHAT_CONFIG_OPTION_NULL,
+                                                         0, WEECHAT_LIST_POS_BEGINNING);
                             }
                             break;
                         case CONFIG_OPTION_TYPE_STRING:
@@ -1335,6 +1609,30 @@ completion_list_add_config_option_values_cb (const void *pointer, void *data,
                                                          0, WEECHAT_LIST_POS_BEGINNING);
                             }
                             break;
+                        case CONFIG_OPTION_TYPE_ENUM:
+                            for (i = 0; option_found->string_values[i]; i++)
+                            {
+                                gui_completion_list_add (completion,
+                                                         option_found->string_values[i],
+                                                         0, WEECHAT_LIST_POS_SORT);
+                            }
+                            gui_completion_list_add (completion, "++1",
+                                                     0, WEECHAT_LIST_POS_END);
+                            gui_completion_list_add (completion, "--1",
+                                                     0, WEECHAT_LIST_POS_END);
+                            if (option_found->value)
+                            {
+                                gui_completion_list_add (completion,
+                                                         option_found->string_values[CONFIG_ENUM(option_found)],
+                                                         0, WEECHAT_LIST_POS_BEGINNING);
+                            }
+                            else
+                            {
+                                gui_completion_list_add (completion,
+                                                         WEECHAT_CONFIG_OPTION_NULL,
+                                                         0, WEECHAT_LIST_POS_BEGINNING);
+                            }
+                            break;
                         case CONFIG_NUM_OPTION_TYPES:
                             break;
                     }
@@ -1478,7 +1776,7 @@ completion_list_add_keys_contexts_cb (const void *pointer, void *data,
                                       struct t_gui_buffer *buffer,
                                       struct t_gui_completion *completion)
 {
-    int i;
+    int context;
 
     /* make C compiler happy */
     (void) pointer;
@@ -1486,9 +1784,9 @@ completion_list_add_keys_contexts_cb (const void *pointer, void *data,
     (void) completion_item;
     (void) buffer;
 
-    for (i = 0; i < GUI_KEY_NUM_CONTEXTS; i++)
+    for (context = 0; context < GUI_KEY_NUM_CONTEXTS; context++)
     {
-        gui_completion_list_add (completion, gui_key_context_string[i],
+        gui_completion_list_add (completion, gui_key_context_string[context],
                                  0, WEECHAT_LIST_POS_END);
     }
 
@@ -1505,9 +1803,8 @@ completion_list_add_keys_codes_cb (const void *pointer, void *data,
                                    struct t_gui_buffer *buffer,
                                    struct t_gui_completion *completion)
 {
-    int i;
+    int context;
     struct t_gui_key *ptr_key;
-    char *expanded_name;
 
     /* make C compiler happy */
     (void) pointer;
@@ -1515,17 +1812,12 @@ completion_list_add_keys_codes_cb (const void *pointer, void *data,
     (void) completion_item;
     (void) buffer;
 
-    for (i = 0; i < GUI_KEY_NUM_CONTEXTS; i++)
+    for (context = 0; context < GUI_KEY_NUM_CONTEXTS; context++)
     {
-        for (ptr_key = gui_keys[i]; ptr_key; ptr_key = ptr_key->next_key)
+        for (ptr_key = gui_keys[context]; ptr_key; ptr_key = ptr_key->next_key)
         {
-            expanded_name = gui_key_get_expanded_name (ptr_key->key);
-            gui_completion_list_add (
-                completion,
-                (expanded_name) ? expanded_name : ptr_key->key,
-                0, WEECHAT_LIST_POS_SORT);
-            if (expanded_name)
-                free (expanded_name);
+            gui_completion_list_add (completion, ptr_key->key,
+                                     0, WEECHAT_LIST_POS_SORT);
         }
     }
 
@@ -1543,9 +1835,8 @@ completion_list_add_keys_codes_for_reset_cb (const void *pointer, void *data,
                                              struct t_gui_buffer *buffer,
                                              struct t_gui_completion *completion)
 {
-    int i;
+    int context;
     struct t_gui_key *ptr_key, *ptr_default_key;
-    char *expanded_name;
 
     /* make C compiler happy */
     (void) pointer;
@@ -1553,39 +1844,31 @@ completion_list_add_keys_codes_for_reset_cb (const void *pointer, void *data,
     (void) completion_item;
     (void) buffer;
 
-    for (i = 0; i < GUI_KEY_NUM_CONTEXTS; i++)
+    for (context = 0; context < GUI_KEY_NUM_CONTEXTS; context++)
     {
         /* keys added or redefined */
-        for (ptr_key = gui_keys[i]; ptr_key; ptr_key = ptr_key->next_key)
+        for (ptr_key = gui_keys[context]; ptr_key; ptr_key = ptr_key->next_key)
         {
-            ptr_default_key = gui_key_search (gui_default_keys[i], ptr_key->key);
+            ptr_default_key = gui_key_search (gui_default_keys[context],
+                                              ptr_key->key);
             if (!ptr_default_key
                 || (strcmp (ptr_default_key->command, ptr_key->command) != 0))
             {
-                expanded_name = gui_key_get_expanded_name (ptr_key->key);
-                gui_completion_list_add (
-                    completion,
-                    (expanded_name) ? expanded_name : ptr_key->key,
-                    0, WEECHAT_LIST_POS_SORT);
-                if (expanded_name)
-                    free (expanded_name);
+                gui_completion_list_add (completion, ptr_key->key,
+                                         0, WEECHAT_LIST_POS_SORT);
             }
         }
 
         /* keys deleted */
-        for (ptr_default_key = gui_default_keys[i]; ptr_default_key;
+        for (ptr_default_key = gui_default_keys[context]; ptr_default_key;
              ptr_default_key = ptr_default_key->next_key)
         {
-            ptr_key = gui_key_search (gui_keys[i], ptr_default_key->key);
+            ptr_key = gui_key_search (gui_keys[context],
+                                      ptr_default_key->key);
             if (!ptr_key)
             {
-                expanded_name = gui_key_get_expanded_name (ptr_default_key->key);
-                gui_completion_list_add (
-                    completion,
-                    (expanded_name) ? expanded_name : ptr_default_key->key,
-                    0, WEECHAT_LIST_POS_SORT);
-                if (expanded_name)
-                    free (expanded_name);
+                gui_completion_list_add (completion, ptr_default_key->key,
+                                         0, WEECHAT_LIST_POS_SORT);
             }
         }
     }
@@ -1788,6 +2071,119 @@ completion_list_add_env_value_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Adds a buffer local variable for /eval to completions list.
+ */
+
+void
+completion_list_map_eval_buffer_local_variable_cb (void *data,
+                                                   struct t_hashtable *hashtable,
+                                                   const void *key, const void *value)
+{
+    char *name;
+    int length;
+
+    /* make C compiler happy */
+    (void) hashtable;
+    (void) value;
+
+    length = strlen (key) + 3 + 1;
+    name = malloc (length);
+    if (name)
+    {
+        snprintf (name, length, "${%s}", (const char *)key);
+        gui_completion_list_add ((struct t_gui_completion *)data,
+                                 name, 0, WEECHAT_LIST_POS_SORT);
+        free (name);
+    }
+}
+
+/*
+ * Adds /eval variables to completion list.
+ */
+
+int
+completion_list_add_eval_variables_cb (const void *pointer, void *data,
+                                       const char *completion_item,
+                                       struct t_gui_buffer *buffer,
+                                       struct t_gui_completion *completion)
+{
+    char *eval_variables[] = {
+        "${\\xxx}",
+        "${base_decode:base,xxx}",
+        "${base_encode:base,xxx}",
+        "${calc:xxx}",
+        "${chars:range}",
+        "${color:xxx}",
+        "${cut:+max,suffix,string}",
+        "${cut:max,suffix,string}",
+        "${cutscr:+max,suffix,string}",
+        "${cutscr:max,suffix,string}",
+        "${date:format}",
+        "${date}",
+        "${define:name,value}",
+        "${env:XXX}",
+        "${esc:xxx}",
+        "${eval:xxx}",
+        "${eval_cond:xxx}",
+        "${file.section.option}",
+        "${hdata.var1.var2}",
+        "${hdata[list].var1.var2}",
+        "${hdata[ptr].var1.var2}",
+        "${hdata[ptr_name].var1.var2}",
+        "${hide:char,string}",
+        "${if:condition?value_if_true:value_if_false}",
+        "${info:name,arguments}",
+        "${length:xxx}",
+        "${lengthscr:xxx}",
+        "${lower:xxx}",
+        "${modifier:name,data,xxx}",
+        "${random:min,max}",
+        "${raw:xxx}",
+        "${re:+}",
+        "${re:N}",
+        "${repeat:count,string}",
+        "${rev:xxx}",
+        "${revscr:xxx}",
+        "${sec.data.xxx}",
+        "${split:count,separators,flags,xxx}",
+        "${split:number,separators,flags,xxx}",
+        "${split:random,separators,flags,xxx}",
+        "${split_shell:count,xxx}",
+        "${split_shell:number,xxx}",
+        "${split_shell:random,xxx}",
+        "${translate:xxx}",
+        "${upper:xxx}",
+        "${weechat_cache_dir}",
+        "${weechat_config_dir}",
+        "${weechat_data_dir}",
+        "${weechat_runtime_dir}",
+        "${window}",
+        "${window.buffer}",
+        "${window.buffer.xxx}",
+        NULL,
+    };
+    int i;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    for (i = 0; eval_variables[i]; i++)
+    {
+        gui_completion_list_add (completion, eval_variables[i],
+                                 0, WEECHAT_LIST_POS_SORT);
+    }
+
+    hashtable_map (completion->buffer->local_variables,
+                   &completion_list_map_eval_buffer_local_variable_cb,
+                   completion);
+
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Adds hooks for completions done by WeeChat core.
  */
@@ -1813,6 +2209,9 @@ completion_init ()
     hook_completion (NULL, "buffer_properties_set",
                      N_("properties that can be set on a buffer"),
                      &completion_list_add_buffer_properties_set_cb, NULL, NULL);
+    hook_completion (NULL, "buffer_properties_setauto",
+                     N_("properties that can be automatically set on a buffer"),
+                     &completion_list_add_buffer_properties_setauto_cb, NULL, NULL);
     hook_completion (NULL, "buffer_properties_get",
                      N_("properties that can be read on a buffer"),
                      &completion_list_add_buffer_properties_get_cb, NULL, NULL);
@@ -1836,6 +2235,12 @@ completion_init ()
     hook_completion (NULL, "filters_names", /* formerly "%F" */
                      N_("names of filters"),
                      &completion_list_add_filters_cb, NULL, NULL);
+    hook_completion (NULL, "filters_names_disabled",
+                     N_("names of disabled filters"),
+                     &completion_list_add_filters_disabled_cb, NULL, NULL);
+    hook_completion (NULL, "filters_names_enabled",
+                     N_("names of enabled filters"),
+                     &completion_list_add_filters_enabled_cb, NULL, NULL);
     hook_completion (NULL, "commands", /* formerly "%h" */
                      N_("commands (weechat and plugins); "
                         "optional argument: prefix to add before the commands"),
@@ -1865,6 +2270,19 @@ completion_init ()
     hook_completion (NULL, "bars_names", /* formerly "%r" */
                      N_("names of bars"),
                      &completion_list_add_bars_names_cb, NULL, NULL);
+    hook_completion (NULL, "custom_bar_items_names",
+                     N_("names of custom bar items"),
+                     &completion_list_add_custom_bar_items_names_cb, NULL, NULL);
+    hook_completion (NULL, "custom_bar_item_conditions",
+                     N_("conditions for custom bar item"),
+                     &completion_list_add_custom_bar_item_conditions_cb, NULL, NULL);
+    hook_completion (NULL, "custom_bar_item_contents",
+                     N_("contents for custom bar item"),
+                     &completion_list_add_custom_bar_item_contents_cb, NULL, NULL);
+    hook_completion (NULL, "custom_bar_item_add_arguments",
+                     N_("arguments for command that adds a custom bar item: "
+                        "item name, conditions, content"),
+                     &completion_list_add_custom_bar_item_add_arguments_cb, NULL, NULL);
     hook_completion (NULL, "config_option_values", /* formerly "%v" */
                      N_("values for a configuration option"),
                      &completion_list_add_config_option_values_cb, NULL, NULL);
@@ -1910,4 +2328,7 @@ completion_init ()
     hook_completion (NULL, "env_value",
                      N_("value of an environment variable"),
                      &completion_list_add_env_value_cb, NULL, NULL);
+    hook_completion (NULL, "eval_variables",
+                     N_("variables that can be used in /eval command"),
+                     &completion_list_add_eval_variables_cb, NULL, NULL);
 }
