@@ -74,6 +74,8 @@ char *gui_mouse_button_codes[][2] =
   { "*",  "alt-button2"        },
   { ")",  "alt-button3"        },
   { "8",  "alt-ctrl-button1"   },
+  { ":",  "alt-ctrl-button2"   },
+  { "9",  "alt-ctrl-button3"   },
   { NULL, NULL                 } };
 
 
