@@ -1,7 +1,7 @@
 /*
  * charset.c - charset plugin for WeeChat: encode/decode strings
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -28,21 +28,17 @@
 #include <iconv.h>
 
 #include "../weechat-plugin.h"
+#include "charset.h"
 
 
-#define CHARSET_PLUGIN_NAME "charset"
-
 WEECHAT_PLUGIN_NAME(CHARSET_PLUGIN_NAME);
 WEECHAT_PLUGIN_DESCRIPTION(N_("Charset conversions"));
 WEECHAT_PLUGIN_AUTHOR("Sébastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
-WEECHAT_PLUGIN_PRIORITY(13000);
-
-#define CHARSET_CONFIG_NAME "charset"
+WEECHAT_PLUGIN_PRIORITY(CHARSET_PLUGIN_PRIORITY);
 
 struct t_weechat_plugin *weechat_charset_plugin = NULL;
-#define weechat_plugin weechat_charset_plugin
 
 struct t_config_file *charset_config_file = NULL;
 struct t_config_option *charset_default_decode = NULL;
@@ -50,8 +46,8 @@ struct t_config_option *charset_default_encode = NULL;
 struct t_config_section *charset_config_section_decode = NULL;
 struct t_config_section *charset_config_section_encode = NULL;
 
-const char *charset_terminal = NULL;
-const char *charset_internal = NULL;
+char *charset_terminal = NULL;
+char *charset_internal = NULL;
 
 
 /*
@@ -59,9 +55,11 @@ const char *charset_internal = NULL;
  */
 
 int
-charset_config_reload (void *data, struct t_config_file *config_file)
+charset_config_reload (const void *pointer, void *data,
+                       struct t_config_file *config_file)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     /* free all decode/encode charsets */
@@ -107,10 +105,12 @@ charset_decode_is_allowed (const char *charset)
  */
 
 int
-charset_check_charset_decode_cb (void *data, struct t_config_option *option,
+charset_check_charset_decode_cb (const void *pointer, void *data,
+                                 struct t_config_option *option,
                                  const char *value)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) option;
 
@@ -122,7 +122,8 @@ charset_check_charset_decode_cb (void *data, struct t_config_option *option,
  */
 
 int
-charset_config_create_option (void *data, struct t_config_file *config_file,
+charset_config_create_option (const void *pointer, void *data,
+                              struct t_config_file *config_file,
                               struct t_config_section *section,
                               const char *option_name, const char *value)
 {
@@ -130,6 +131,7 @@ charset_config_create_option (void *data, struct t_config_file *config_file,
     int rc;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     rc = WEECHAT_CONFIG_OPTION_SET_ERROR;
@@ -159,8 +161,9 @@ charset_config_create_option (void *data, struct t_config_file *config_file,
                         config_file, section,
                         option_name, "string", NULL,
                         NULL, 0, 0, "", value, 0,
-                        (section == charset_config_section_decode) ? &charset_check_charset_decode_cb : NULL, NULL,
-                        NULL, NULL, NULL, NULL);
+                        (section == charset_config_section_decode) ? &charset_check_charset_decode_cb : NULL, NULL, NULL,
+                        NULL, NULL, NULL,
+                        NULL, NULL, NULL);
                     rc = (ptr_option) ?
                         WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE : WEECHAT_CONFIG_OPTION_SET_ERROR;
                 }
@@ -194,19 +197,23 @@ charset_config_init ()
 {
     struct t_config_section *ptr_section;
 
-    charset_config_file = weechat_config_new (CHARSET_CONFIG_NAME,
-                                              &charset_config_reload, NULL);
+    charset_config_file = weechat_config_new (
+        CHARSET_CONFIG_PRIO_NAME,
+        &charset_config_reload, NULL, NULL);
     if (!charset_config_file)
         return 0;
 
     ptr_section = weechat_config_new_section (charset_config_file, "default",
                                               0, 0,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL);
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL);
     if (!ptr_section)
     {
         weechat_config_free (charset_config_file);
+        charset_config_file = NULL;
         return 0;
     }
 
@@ -220,7 +227,9 @@ charset_config_init ()
          && (weechat_strcasecmp (charset_terminal,
                                  charset_internal) != 0)) ?
         charset_terminal : "iso-8859-1", NULL, 0,
-        &charset_check_charset_decode_cb, NULL, NULL, NULL, NULL, NULL);
+        &charset_check_charset_decode_cb, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL);
     charset_default_encode = weechat_config_new_option (
         charset_config_file, ptr_section,
         "encode", "string",
@@ -228,31 +237,37 @@ charset_config_init ()
            "(if empty, default is UTF-8 because it is the WeeChat internal "
            "charset)"),
         NULL, 0, 0, "", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 
-    ptr_section = weechat_config_new_section (charset_config_file, "decode",
-                                              1, 1,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL,
-                                              &charset_config_create_option, NULL,
-                                              NULL, NULL);
+    ptr_section = weechat_config_new_section (
+        charset_config_file, "decode",
+        1, 1,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL,
+        &charset_config_create_option, NULL, NULL,
+        NULL, NULL, NULL);
     if (!ptr_section)
     {
         weechat_config_free (charset_config_file);
+        charset_config_file = NULL;
         return 0;
     }
 
     charset_config_section_decode = ptr_section;
 
-    ptr_section = weechat_config_new_section (charset_config_file, "encode",
-                                              1, 1,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL,
-                                              &charset_config_create_option, NULL,
-                                              NULL, NULL);
+    ptr_section = weechat_config_new_section (
+        charset_config_file, "encode",
+        1, 1,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL,
+        &charset_config_create_option, NULL, NULL,
+        NULL, NULL, NULL);
     if (!ptr_section)
     {
         weechat_config_free (charset_config_file);
+        charset_config_file = NULL;
         return 0;
     }
 
@@ -365,12 +380,14 @@ charset_get (struct t_config_section *section, const char *name,
  */
 
 char *
-charset_decode_cb (void *data, const char *modifier, const char *modifier_data,
+charset_decode_cb (const void *pointer, void *data,
+                   const char *modifier, const char *modifier_data,
                    const char *string)
 {
     const char *charset;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) modifier;
 
@@ -394,12 +411,14 @@ charset_decode_cb (void *data, const char *modifier, const char *modifier_data,
  */
 
 char *
-charset_encode_cb (void *data, const char *modifier, const char *modifier_data,
+charset_encode_cb (const void *pointer, void *data,
+                   const char *modifier, const char *modifier_data,
                    const char *string)
 {
     const char *charset;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) modifier;
 
@@ -426,7 +445,7 @@ void
 charset_set (struct t_config_section *section, const char *type,
              const char *name, const char *value)
 {
-    if (charset_config_create_option (NULL,
+    if (charset_config_create_option (NULL, NULL,
                                       charset_config_file,
                                       section,
                                       name,
@@ -458,7 +477,8 @@ charset_display_charsets ()
  */
 
 int
-charset_command_cb (void *data, struct t_gui_buffer *buffer, int argc,
+charset_command_cb (const void *pointer, void *data,
+                    struct t_gui_buffer *buffer, int argc,
                     char **argv, char **argv_eol)
 {
     struct t_config_section *ptr_section;
@@ -467,6 +487,7 @@ charset_command_cb (void *data, struct t_gui_buffer *buffer, int argc,
     const char *plugin_name, *name, *charset_modifier;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     if (argc < 2)
@@ -494,7 +515,7 @@ charset_command_cb (void *data, struct t_gui_buffer *buffer, int argc,
         snprintf (option_name, length, "%s.%s", plugin_name, name);
     }
 
-    if (weechat_strcasecmp (argv[1], "reset") == 0)
+    if (weechat_strcmp (argv[1], "reset") == 0)
     {
         charset_set (charset_config_section_decode, "decode", option_name,
                      NULL);
@@ -505,12 +526,12 @@ charset_command_cb (void *data, struct t_gui_buffer *buffer, int argc,
     {
         if (argc > 2)
         {
-            if (weechat_strcasecmp (argv[1], "decode") == 0)
+            if (weechat_strcmp (argv[1], "decode") == 0)
             {
                 ptr_section = charset_config_section_decode;
                 ptr_charset = argv_eol[2];
             }
-            else if (weechat_strcasecmp (argv[1], "encode") == 0)
+            else if (weechat_strcmp (argv[1], "encode") == 0)
             {
                 ptr_section = charset_config_section_encode;
                 ptr_charset = argv_eol[2];
@@ -587,18 +608,20 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     weechat_hook_command (
         "charset",
         N_("change charset for current buffer"),
+        /* TRANSLATORS: only text between angle brackets (eg: "<name>") must be translated */
         N_("decode|encode <charset>"
            " || reset"),
-        N_(" decode: change decoding charset\n"
-           " encode: change encoding charset\n"
-           "charset: new charset for current buffer\n"
-           "  reset: reset charsets for current buffer"),
+        WEECHAT_CMD_ARGS_DESC(
+            N_("raw[decode]: change decoding charset"),
+            N_("raw[encode]: change encoding charset"),
+            N_("charset: new charset for current buffer"),
+            N_("raw[reset]: reset charsets for current buffer")),
         "decode|encode|reset",
-        &charset_command_cb, NULL);
+        &charset_command_cb, NULL, NULL);
 
     /* modifiers hooks */
-    weechat_hook_modifier ("charset_decode", &charset_decode_cb, NULL);
-    weechat_hook_modifier ("charset_encode", &charset_encode_cb, NULL);
+    weechat_hook_modifier ("charset_decode", &charset_decode_cb, NULL, NULL);
+    weechat_hook_modifier ("charset_encode", &charset_encode_cb, NULL, NULL);
 
     return WEECHAT_RC_OK;
 }
@@ -616,6 +639,18 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
     charset_config_write ();
 
     weechat_config_free (charset_config_file);
+    charset_config_file = NULL;
+
+    if (charset_terminal)
+    {
+        free (charset_terminal);
+        charset_terminal = NULL;
+    }
+    if (charset_internal)
+    {
+        free (charset_internal);
+        charset_internal = NULL;
+    }
 
     return WEECHAT_RC_OK;
 }
