@@ -1,7 +1,7 @@
 /*
  * irc-redirect.c - redirection of IRC command output
  *
- * Copyright (C) 2010-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2010-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -55,7 +55,7 @@ struct t_irc_redirect_pattern irc_redirect_patterns_default[] =
        *        stop: 323: end of /list
        *       extra: -
        */
-      "321",
+      "321,322",
       "323",
       NULL,
       NULL, NULL,
@@ -325,7 +325,7 @@ irc_redirect_pattern_new (const char *name, int temp_pattern, int timeout,
 
     /* add redirect pattern to end of list */
     new_redirect_pattern->prev_redirect = last_irc_redirect_pattern;
-    if (irc_redirect_patterns)
+    if (last_irc_redirect_pattern)
         last_irc_redirect_pattern->next_redirect = new_redirect_pattern;
     else
         irc_redirect_patterns = new_redirect_pattern;
@@ -406,7 +406,7 @@ irc_redirect_new_with_commands (struct t_irc_server *server,
                                 const char *cmd_filter)
 {
     struct t_irc_redirect *new_redirect;
-    char **items[4], *pos, *error;
+    char **items[4], *item_upper, *pos, *error;
     int i, j, num_items[4];
     long value;
     struct t_hashtable *hash_cmd[4];
@@ -422,13 +422,29 @@ irc_redirect_new_with_commands (struct t_irc_server *server,
         items[i] = NULL;
     }
     if (cmd_start)
-        items[0] = weechat_string_split (cmd_start, ",", 0, 0, &num_items[0]);
+        items[0] = weechat_string_split (cmd_start, ",", NULL,
+                                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                         0, &num_items[0]);
     if (cmd_stop)
-        items[1] = weechat_string_split (cmd_stop, ",", 0, 0, &num_items[1]);
+        items[1] = weechat_string_split (cmd_stop, ",", NULL,
+                                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                         0, &num_items[1]);
     if (cmd_extra)
-        items[2] = weechat_string_split (cmd_extra, ",", 0, 0, &num_items[2]);
+        items[2] = weechat_string_split (cmd_extra, ",", NULL,
+                                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                         0, &num_items[2]);
     if (cmd_filter)
-        items[3] = weechat_string_split (cmd_filter, ",", 0, 0, &num_items[3]);
+        items[3] = weechat_string_split (cmd_filter, ",", NULL,
+                                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                         0, &num_items[3]);
     for (i = 0; i < 4; i++)
     {
         if (items[i])
@@ -436,8 +452,7 @@ irc_redirect_new_with_commands (struct t_irc_server *server,
             hash_cmd[i] = weechat_hashtable_new (32,
                                                  WEECHAT_HASHTABLE_STRING,
                                                  WEECHAT_HASHTABLE_INTEGER,
-                                                 NULL,
-                                                 NULL);
+                                                 NULL, NULL);
             for (j = 0; j < num_items[i]; j++)
             {
                 if (i < 3)
@@ -447,12 +462,17 @@ irc_redirect_new_with_commands (struct t_irc_server *server,
                     if (pos)
                     {
                         pos[0] = '\0';
+                        error = NULL;
                         value = strtol (pos + 1, &error, 10);
                         if (!error || error[0])
                             value = -1;
                     }
-                    weechat_string_toupper (items[i][j]);
-                    weechat_hashtable_set (hash_cmd[i], items[i][j], &value);
+                    item_upper = weechat_string_toupper (items[i][j]);
+                    if (item_upper)
+                    {
+                        weechat_hashtable_set (hash_cmd[i], item_upper, &value);
+                        free (item_upper);
+                    }
                 }
                 else
                 {
@@ -485,7 +505,7 @@ irc_redirect_new_with_commands (struct t_irc_server *server,
 
     /* add redirect to end of list */
     new_redirect->prev_redirect = server->last_redirect;
-    if (server->redirects)
+    if (server->last_redirect)
         (server->last_redirect)->next_redirect = new_redirect;
     else
         server->redirects = new_redirect;
@@ -687,7 +707,7 @@ irc_redirect_message_add (struct t_irc_redirect *redirect, const char *message,
     /* add message to output */
     if (redirect->output)
     {
-        redirect->output_size += strlen("\n") + strlen (message);
+        redirect->output_size += strlen ("\n") + strlen (message);
         output2 = realloc (redirect->output, redirect->output_size);
         if (!output2)
         {
@@ -732,8 +752,7 @@ irc_redirect_stop (struct t_irc_redirect *redirect, const char *error)
         hashtable = weechat_hashtable_new (32,
                                            WEECHAT_HASHTABLE_STRING,
                                            WEECHAT_HASHTABLE_STRING,
-                                           NULL,
-                                           NULL);
+                                           NULL, NULL);
         if (hashtable)
         {
             /* set error and output (main fields) */
@@ -795,8 +814,15 @@ irc_redirect_message (struct t_irc_server *server, const char *message,
 
     if (arguments && arguments[0])
     {
-        arguments_argv = weechat_string_split (arguments, " ", 0, 0,
-                                               &arguments_argc);
+        arguments_argv = weechat_string_split (
+            arguments,
+            " ",
+            NULL,
+            WEECHAT_STRING_SPLIT_STRIP_LEFT
+            | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+            | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+            0,
+            &arguments_argc);
     }
     else
     {
@@ -861,7 +887,7 @@ irc_redirect_message (struct t_irc_server *server, const char *message,
                 if (match_stop || ptr_redirect->cmd_start_received)
                 {
                     /*
-                     * add message to output if matching stop of if command
+                     * add message to output if matching stop or if command
                      * is numeric
                      */
                     irc_redirect_message_add (ptr_redirect, message, command);
@@ -993,11 +1019,13 @@ irc_redirect_free_all (struct t_irc_server *server)
  */
 
 struct t_hdata *
-irc_redirect_hdata_redirect_pattern_cb (void *data, const char *hdata_name)
+irc_redirect_hdata_redirect_pattern_cb (const void *pointer, void *data,
+                                        const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = weechat_hdata_new (hdata_name, "prev_redirect", "next_redirect",
@@ -1023,11 +1051,13 @@ irc_redirect_hdata_redirect_pattern_cb (void *data, const char *hdata_name)
  */
 
 struct t_hdata *
-irc_redirect_hdata_redirect_cb (void *data, const char *hdata_name)
+irc_redirect_hdata_redirect_cb (const void *pointer, void *data,
+                                const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = weechat_hdata_new (hdata_name, "prev_redirect", "next_redirect",
@@ -1207,7 +1237,7 @@ irc_redirect_print_log (struct t_irc_server *server)
         weechat_log_printf ("       timeout . . . . . . : %d",    ptr_redirect->timeout);
         weechat_log_printf ("       command . . . . . . : '%s'",  ptr_redirect->command);
         weechat_log_printf ("       assigned_to_command : %d",    ptr_redirect->assigned_to_command);
-        weechat_log_printf ("       start_time. . . . . : %ld",   ptr_redirect->start_time);
+        weechat_log_printf ("       start_time. . . . . : %lld",  (long long)ptr_redirect->start_time);
         weechat_log_printf ("       cmd_start . . . . . : 0x%lx (hashtable: '%s')",
                             ptr_redirect->cmd_start,
                             weechat_hashtable_get_string (ptr_redirect->cmd_start, "keys_values"));
@@ -1237,7 +1267,8 @@ irc_redirect_print_log (struct t_irc_server *server)
  */
 
 int
-irc_redirect_pattern_hsignal_cb (void *data, const char *signal,
+irc_redirect_pattern_hsignal_cb (const void *pointer, void *data,
+                                 const char *signal,
                                  struct t_hashtable *hashtable)
 {
     const char *pattern, *str_timeout, *cmd_start, *cmd_stop, *cmd_extra;
@@ -1245,6 +1276,7 @@ irc_redirect_pattern_hsignal_cb (void *data, const char *signal,
     int number, timeout;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) signal;
 
@@ -1278,6 +1310,7 @@ irc_redirect_pattern_hsignal_cb (void *data, const char *signal,
     timeout = 0;
     if (str_timeout && str_timeout[0])
     {
+        error = NULL;
         number = (int)strtol (str_timeout, &error, 10);
         if (error && !error[0])
             timeout = number;
@@ -1301,7 +1334,8 @@ irc_redirect_pattern_hsignal_cb (void *data, const char *signal,
  */
 
 int
-irc_redirect_command_hsignal_cb (void *data, const char *signal,
+irc_redirect_command_hsignal_cb (const void *pointer, void *data,
+                                 const char *signal,
                                  struct t_hashtable *hashtable)
 {
     const char *server, *pattern, *redirect_signal, *str_count, *string;
@@ -1311,6 +1345,7 @@ irc_redirect_command_hsignal_cb (void *data, const char *signal,
     int number, count, timeout;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) signal;
 
@@ -1346,6 +1381,7 @@ irc_redirect_command_hsignal_cb (void *data, const char *signal,
     count = 1;
     if (str_count && str_count[0])
     {
+        error = NULL;
         number = (int)strtol (str_count, &error, 10);
         if (error && !error[0])
             count = number;
@@ -1354,6 +1390,7 @@ irc_redirect_command_hsignal_cb (void *data, const char *signal,
     timeout = 0;
     if (str_timeout && str_timeout[0])
     {
+        error = NULL;
         number = (int)strtol (str_timeout, &error, 10);
         if (error && !error[0])
             timeout = number;
