@@ -1,7 +1,7 @@
 /*
  * plugin-config.c - plugin configuration options (file plugins.conf)
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -114,7 +114,7 @@ plugin_config_set (const char *plugin_name, const char *option_name,
                    const char *value)
 {
     int length, rc;
-    char *option_full_name, *option_full_name_lower;
+    char *option_full_name;
 
     rc = WEECHAT_CONFIG_OPTION_SET_ERROR;
 
@@ -124,12 +124,7 @@ plugin_config_set (const char *plugin_name, const char *option_name,
     {
         snprintf (option_full_name, length, "%s.%s",
                   plugin_name, option_name);
-        option_full_name_lower = string_tolower (option_full_name);
-        if (option_full_name_lower)
-        {
-            rc = plugin_config_set_internal (option_full_name_lower, value);
-            free (option_full_name_lower);
-        }
+        rc = plugin_config_set_internal (option_full_name, value);
         free (option_full_name);
     }
 
@@ -206,7 +201,7 @@ plugin_config_set_desc (const char *plugin_name, const char *option_name,
                         const char *description)
 {
     int length;
-    char *option_full_name, *option_full_name_lower;
+    char *option_full_name;
 
     length = strlen (plugin_name) + 1 + strlen (option_name) + 1;
     option_full_name = malloc (length);
@@ -214,13 +209,7 @@ plugin_config_set_desc (const char *plugin_name, const char *option_name,
     {
         snprintf (option_full_name, length, "%s.%s",
                   plugin_name, option_name);
-        option_full_name_lower = string_tolower (option_full_name);
-        if (option_full_name_lower)
-        {
-            plugin_config_set_desc_internal (option_full_name_lower,
-                                             description);
-            free (option_full_name_lower);
-        }
+        plugin_config_set_desc_internal (option_full_name, description);
         free (option_full_name);
     }
 }
